@@ -95,7 +95,7 @@ def compile_plain(source, options, keep, nvcc, cache_dir, target="cubin"):
             finally:
                 cache_file.close()
 
-        except:
+        except Exception:
             pass
 
     from tempfile import mkdtemp
@@ -221,7 +221,6 @@ def compile(source, nvcc="nvcc", options=None, keep=False,
         cache_dir = False
 
     if cache_dir is None:
-        from os.path import join
         import appdirs
         cache_dir = os.path.join(appdirs.user_cache_dir("pycuda", "pycuda"),
                 "compiler-cache-v1")
@@ -254,6 +253,7 @@ def compile(source, nvcc="nvcc", options=None, keep=False,
 
     return compile_plain(source, options, keep, nvcc, cache_dir, target)
 
+
 class CudaModule(object):
     def _check_arch(self, arch):
         if arch is None:
@@ -265,7 +265,7 @@ class CudaModule(object):
                 from warnings import warn
                 warn("trying to compile for a compute capability "
                         "higher than selected GPU")
-        except:
+        except Exception:
             pass
 
     def _bind_module(self):
@@ -393,13 +393,14 @@ class DynamicModule(CudaModule):
                         libdir = ld_path
                         break
 
+            if libdir is None and isfile('/usr/lib/x86_64-linux-gnu/libcudadevrt.a'):
+                libdir = '/usr/lib/x86_64-linux-gnu'
+
             if libdir is None:
                 nvcc_path = _find_nvcc_on_path()
                 if nvcc_path is not None:
                     libdir = join(os.path.dirname(nvcc_path), "..", "lib64")
 
-            if libdir is None and isfile('/usr/lib/x86_64-linux-gnu/libcudadevrt.a'):
-                libdir = '/usr/lib/x86_64-linux-gnu'
             libptn = 'lib%s.a'
         if libdir is None:
             raise RuntimeError('Unable to locate the CUDA SDK installation '
@@ -429,7 +430,7 @@ class DynamicModule(CudaModule):
         from os.path import isfile, join
         libpath = join(self.libdir, self.libptn % libname)
         if not isfile(libpath):
-            raise FileNotFoundError('CUDA SDK library file "%s" not found' % libpath)
+            raise OSError('CUDA SDK library file "%s" not found' % libpath)
         from pycuda.driver import jit_input_type
         self.linker.add_file(libpath, jit_input_type.LIBRARY)
         return self
@@ -450,13 +451,15 @@ class DynamicSourceModule(DynamicModule):
     - source is linked against the CUDA device runtime library cudadevrt
     - library cudadevrt is statically linked into the generated Module
     '''
-    def __init__(self, source, nvcc="nvcc", options=[], keep=False,
+    def __init__(self, source, nvcc="nvcc", options=None, keep=False,
             no_extern_c=False, arch=None, code=None, cache_dir=None,
             include_dirs=[], cuda_libdir=None):
         super(DynamicSourceModule, self).__init__(nvcc=nvcc,
             link_options=None, keep=keep, no_extern_c=no_extern_c,
             arch=arch, code=code, cache_dir=cache_dir,
             include_dirs=include_dirs, cuda_libdir=cuda_libdir)
+        if options is None:
+            options = DEFAULT_NVCC_FLAGS
         options = options[:]
         if '-rdc=true' not in options:
             options.append('-rdc=true')
